/* eslint-env node */

const jhipster = require('jhipster-core'); 
const inflection = require('inflection');
const shelljs = require('shelljs');


module.exports = {
  name: 'jdlimporter',
  fileName: '',
  description: '',
  SEPARATOR: ';',
  entities: {},

  normalizeEntityName(entityName) {
    // Normalize and validate entity name here.
    this.fileName = entityName;
    return entityName;
  },

  beforeInstall(options) {
    this.ui.writeLine("afterInstall processing for jdlimporter module with file "+this.fileName);
    // TODO Check options contains a link on a JDL file

    // Parse provided JDL file
    var files= this.fileName.split(this.SEPARATOR);
    var jdlModel = jhipster.parseFromFiles(files);
    //this.ui.writeLine(JSON.stringify(jdlModel));

    let entities = jdlModel.entities;
    let relationships = jdlModel.relationships;

    for (var i=0; i<entities.length;i++) {
      let entity = entities[i];
      // Add an array field to further hold options to pass to entity-factory blueprint
      entity.options = [];
      for (var j=0; j<entity.body.length;j++) {
        let property = entity.body[j];
        switch (property.type) {
          case 'String':
          case 'TextBlob':
            entity.options.push(property.name+':string');
            break;

          case 'LocalDate':
          case 'Instant':
            entity.options.push(property.name+':date');
            break;

          case 'Integer':
          case 'Float':
          case 'Double':
            entity.options.push(property.name+':number');
            break;

          default:
            this.ui.writeLine("Huston, we have a problem with unknow type "+property.name);
            break;
        }
      }
      // Check if current object is involved in a relationship
      // First see the 'from' relationships
      relationships.filter(relation => relation.from.name == entity.name).forEach(relation => {
      let toObjectName = relation.to.name.toLowerCase();
        switch (relation.cardinality) {
          case "one-to-one":
          case "many-to-one":
            entity.options.push(toObjectName+':belongs-to:'+toObjectName);
            break;

          case "one-to-many":
          case "many-to-many":
            entity.options.push(inflection.pluralize(toObjectName)+':has-many:'+toObjectName);
            break;

          default:
            this.ui.writeLine("Huston, we have a problem with unknow relationship type "+relation.cardinality);
            break;
        }
       });
       // Do the same for to relationships
       relationships.filter(relation => relation.to.name == entity.name).forEach(relation => {
       let toObjectName = relation.from.name.toLowerCase();
          switch (relation.cardinality) {
            case "one-to-one":
            case "one-to-many":
              entity.options.push(toObjectName+':has-many:'+toObjectName);
              break;
  
            case "many-to-one":  
            case "many-to-many":
              entity.options.push(inflection.pluralize(toObjectName)+':has-many:'+toObjectName);
              break;
  
            default:
              this.ui.writeLine("Huston, we have a problem with unknow relationship type "+relation.cardinality);
              break;
          }
        });
        console.log('ember g entity-factory ' + entity.name.toLowerCase() + ' ' + entity.options.join(" "));
    }

  },

  locals: function(options) {
  },
  
  afterInstall(options) {
    this.ui.writeLine("Real stuff begins here !");
     // Check ENV value to add AWS stuff

     // Perform extra work here.
  }
};

